/*************************************************************************************
Data Visualization - 2D network map of Travel library items in Seattle Public Library and their connection to othertravel library items            
Tested in Processing 3                                    
                                                     
Author: Christina Last (sections of code provided by Rodger Luo)                      
Supervisor: Rodger Luo                           

Purpose: To show the different dewey classes associated with travel and the amount of location based library items.

Usage:    1. Hovering over the title names will highlight only their links to other nodes that share the same dewey class
          2. Pressing the mouse will bring the original visualisation of all dewey class links
*************************************************************************************/


Table table;
int numRows;
float[] radii;
String[] titles;
float[] dewey;
String[] deweyType = {
  "Travel in Australasia and the Pacific Ocean, and other",
  "Historical Geography",
  "Cartography",
  "Travel in the Ancient World",
  "Travel in Europe",
  "Travel in Asia",
  "Travel in Africa",
  "Travel in North and Central America",
  "Travel in South America",
  "Geography and Travel"
};

float[] xPositions;
float yPosition;
float circlesXFromLeft = 20;
float circlesYFromBottom = 220;
float circlesXSpace = 1280 - 2 * circlesXFromLeft;
float circlesEffectiveWidth = 0;
float gapBetweenTextAndCircle = 20;

boolean mouseOn = false;
int selectedIndex;

void setup() {
  size(1300,1000);
  colorMode(RGB, 700);

  table = loadTable("012818_travel_dewey_100.csv");
  numRows = table.getRowCount() - 1;

  radii = new float[numRows];
  xPositions = new float[numRows];
  titles = new String[numRows];
  dewey = new float[numRows];

  for (int i = 0; i < numRows; i++) {
    // radii
    radii[i] = float(table.getInt(i + 1, 0)) * 0.6;
    circlesEffectiveWidth = circlesEffectiveWidth + radii[i] * 2;

    // dewey
    // dewey[i] = table.getFloat(i + 1, 1);
    dewey[i] = table.getFloat(i + 1, 1) % 1000;  // hack, to get the key display working

    // title
    titles[i] = table.getString(i + 1, 2);
  }
  
  // recalculate xPositions & radii based on total width
  for (int i = 0; i < numRows; i++) {
    radii[i] = radii[i] / circlesEffectiveWidth * circlesXSpace;
    
    if (i == 0) {
      xPositions[0] = circlesXFromLeft + radii[0];
    } else {
      xPositions[i] = xPositions[i - 1] + radii[i - 1] + radii[i];
    }
  }
  yPosition = height - circlesYFromBottom - gapBetweenTextAndCircle;
}

void draw() {
  background(100);

  drawTextsAndLinesOnLeftSide();

  drawTitles();
  
  drawArcs();

  drawCircles();
}

void drawTextsAndLinesOnLeftSide() {
  fill(400);
  strokeWeight(2);
  textSize(10);
  textAlign(LEFT);
  
  float linesStartPosX = 0;
  float linesStartPosY = 20;
  float linesLength = 30;
  float linesGapSpace = 20;
  float textStartPosX = 35;
  float textStartPosY = 23;
  
  for (int i = 0; i < 10; i++) {
    stroke(100 * i, 450, 450, 900);  // set various colours

    if (!mouseOn || i == dewey[selectedIndex] / 100) {
      line(
        linesStartPosX,
        linesStartPosY + linesGapSpace * i,
        linesLength,
        linesStartPosY + linesGapSpace * i
      );
      text(deweyType[i], textStartPosX, textStartPosY + linesGapSpace * i);
    }

    if (mouseOn && i == dewey[selectedIndex] / 100) {
      line(
        linesStartPosX + 70,
        linesStartPosY + linesGapSpace * i + 10,
        linesStartPosX + 70,
        linesStartPosY + linesGapSpace * i + 20
      );
      text(
        titles[selectedIndex],
        linesStartPosX + 60,
        linesStartPosY + linesGapSpace * i + 38
      );
    }
  }
}

void drawTitles() {
  textAlign(LEFT);
  textSize(10);
  
  pushMatrix();
  translate(width / 2, height / 2);  // rotate around center
  rotate(PI / 2);
  
  for (int i = 0; i < numRows; i++) {
    text(
      titles[i],
      height / 2 - circlesYFromBottom,
      width / 2 - xPositions[i] + 2
    );
  }
  
  popMatrix();
}

void drawArcs() {
  noFill();
  
  if (mouseOn) {
    stroke(dewey[selectedIndex], 450, 450, 200);
    
    for (int i = 0; i < numRows; i++) {
      if (dewey[i] == dewey[selectedIndex]) {
        arc(
          (xPositions[i] + xPositions[selectedIndex]) / 2,
          yPosition,
          dist(xPositions[i], yPosition, xPositions[selectedIndex], yPosition),
          dist(xPositions[i], yPosition, xPositions[selectedIndex], yPosition),
          PI,
          PI * 2
        );        
      }
    }
  } else {
    stroke(200, 450, 450, 200);
    
    for (int i = 0; i < numRows; i++) {
      for (int j = 0; j < i; j++) {
        if (dewey[i] == dewey[j]) {
          arc(
            (xPositions[i] + xPositions[j]) / 2,
            yPosition,
            dist(xPositions[i], yPosition, xPositions[j], yPosition),
            dist(xPositions[i], yPosition, xPositions[j], yPosition),
            PI,
            PI * 2
          );
        }
      }
    }    
  }
}

void drawCircles() {
  
  strokeWeight(1);
  stroke(0, 100);
  
  for (int i = 0; i < numRows; i++) {
    if (mouseOn && dewey[i] == dewey[selectedIndex]) {
      fill(dewey[i], 450, 450, 200);
    } else {
      fill(900);
    }

    ellipse(
      xPositions[i],
      yPosition,
      radii[i] * 2,
      radii[i] * 2
    );
  }
}

void keyPressed() {
  if (key == 's') {
    saveFrame("screens/screen-#####.png");
  }
}

void mousePressed() {
  mouseOn = false;
  saveFrame("###.png");
}

void mouseMoved() {
  for (int i = 0; i < numRows; i++) {
    if (dist(mouseX, mouseY, xPositions[i], yPosition) < radii[i]) {
      mouseOn = true;
      selectedIndex = i;
    }
  }
}